/****************************************************************************
    TECNICAS DE PLANIFICACION -- 3 INGENIERIA TECNICA INFORMATICA

Autor: Oscar Martnez Valpuesta
Fecha: semana 14-18 Diciembre 1998
Objetivo: Resolver los ejercicios planteados.
  1. Rutina que saque en pantalla en modo grfico una permutacin de una
     secuencia de trabajos.
  2. Resolver los problemas:
     a) 1 || F_
     b) 1 || W_
     c) 1 || C_
     d) 1 || L_
  3. Resolver el problema 1 || Lmax
  4. Resolver el problema 1 || Uj
  5. Generar aleatoriamente procesos con pj y dj.
  6. Resolver los problemas anteriores con los procesos aleatorios.
*****************************************************************************/
#include "util.c"
#include "menuc.c"
#include "graficas.c"
#include <time.h>

/****************************** CONSTANTES *********************************/
float TF[] = {0.2, 0.4, 0.6, 0.8, 1};
float RDD[] ={0.2, 0.4, 0.6, 0.8};
int num[] = {5, 10, 15, 20, 30, 40, 50, 75, 100, 125, 150};

/********************************** TIPOS **********************************/
typedef struct TIPO_TRABAJO {
  int n_t;		/* n de trabajo */
  int p;		/* tiempo de proceso */
  int d;		/* fecha limite */
  int c;		/* tiempo de finalizacion */
  int flag;		/* campo que indica si el trabajo est en la lista
			   de trabajos para el algortimo de Moore y Hodgson */
} TRABAJO;


/************************** PROTOTIPOS *************************************/
void Menu_principal(unsigned opcion);
void Memoria(void);
void Teclado(void);
void Fichero(void);
void Ejercicio1(void);
void OrdenarSPT(void);
void Ejercicio2(int ver);
void OrdenarEDD(void);
void Ejercicio3(int ver);
void Ejercicio4(int ver);
void Memoria2(void);
void Preparar_fichero(int n, int flag);
void Aleatorios(float tf, float rdd, int P);
void Ejercicio5(void);
void Ejercicio6(void);

/****************************** VARIABLES **********************************/
char *opciones[8]={"7",
		   "Salir",
		   "Diagrama de Gantt",
		   "Resolver problemas equivalentes",
		   "Resolver 1 || Lmax",
		   "Resolver 1 || Uj",
		   "Generar trabajos aleatorios",
		   "Resolver con trabajos aleatorios"};
char *opciones_entrada[4]={"3",
			   "Volver",
			   "Introducir datos por teclado",
			   "Introducir datos por fichero"};

int n;			/* n de trabajos */
TRABAJO *trabajos;	/* vector de trabajos */
int *p;			/* vector con los tiempos de proceso */
int *d;			/* vector con las fechas lmite */
FILE *fichero;		/* descriptor de fichero */
int semilla;		/* semilla para la inicializacin aleatoria */
FILE *out;		/* descriptor para el fichero de salida */
clock_t start, end;	/* instante inicial y final de computacin de un */
			/* algoritmo */
int Lmax;  		/* Lmax para ejercicio 3 */
int u;			/* n de trabajos demorados Uj */
float v[4];		/* [C,F,W,L] medios */

/***************************************************************************/
void Menu_principal(unsigned opcion) {
unsigned opc;

  clrscr();
  switch(opcion) {
    case 1: /* Diagrama de Gantt */
	    opc = Menu(opciones_entrada);
	    /* lectura de las variables por teclado o fichero */
	    switch(opc) {
	      case 1: Teclado();
		      break;
	      case 2: clrscr();
		      fichero = Abrir_fichero();
		      Fichero();
		      break;
	      case 0: break;
	      default: printf("\nERROR");
		       Pausa();
		       break;
	    }
	    if (opc != 1 && opc != 2)
	      break;
	    Ejercicio1();
	    break;
    case 2: /* Problemas equivalentes */
	    opc = Menu(opciones_entrada);
	    /* lectura de las variables por teclado o fichero */
	    switch(opc) {
	      case 1: Teclado();
		      break;
	      case 2: clrscr();
		      fichero = Abrir_fichero();
		      Fichero();
		      break;
	      case 0: break;
	      default: printf("\nERROR");
		       Pausa();
		       break;
	    }
	    if (opc != 1 && opc != 2)
	      break;
	    Ejercicio2(1);
	    break;
    case 3: /* 1 || Lmax */
	    opc = Menu(opciones_entrada);
	    /* lectura de las variables por teclado o fichero */
	    switch(opc) {
	      case 1: Teclado();
		      break;
	      case 2: clrscr();
		      fichero = Abrir_fichero();
		      Fichero();
		      break;
	      case 0: break;
	      default: printf("\nERROR");
		       Pausa();
		       break;
	    }
	    if (opc != 1 && opc != 2)
	      break;
	    Ejercicio3(1);
	    break;
    case 4: /* 1 || Uj */
	    opc = Menu(opciones_entrada);
	    /* lectura de las variables por teclado o fichero */
	    switch(opc) {
	      case 1: Teclado();
		      break;
	      case 2: clrscr();
		      fichero = Abrir_fichero();
		      Fichero();
		      break;
	      case 0: break;
	      default: printf("\nERROR");
		       Pausa();
		       break;
	    }
	    if (opc != 1 && opc != 2)
	      break;
	    Ejercicio4(1);
	    break;
    case 5: Ejercicio5();
	    break;
    case 6: /* resultados finales */
	    clrscr();
	    out = Crear_fichero();
	    printf("Semilla de los ficheros de datos: ");
	    scanf("%d", &semilla);
	    Ejercicio6();
	    break;
    case 0: break;
    default: printf("\nERROR");
	     Pausa();
	     break;
  }

}

/***************************************************************************/
/* Funcion que asigan memoria al vector de trabajos */
void Memoria(void) {

  /* tiempos de proceso */
  if ((trabajos = (TRABAJO *) calloc(n, sizeof(TRABAJO))) == NULL) {
    printf("\nERROR al intentar asignar memoria al vector de trabajos.");
    exit(1);
  }
}

/***************************************************************************/
/* Funcion que lee por teclado los datos para el ejercicios*/
void Teclado(void) {
register int j;

  clrscr();
  printf("Introduce el n de trabajos: ");
  scanf("%d", &n);
  Memoria();

  for (j = 1; j <= n; j++) {
    printf("\nIntroduce el tiempo de proceso del trabajo %d: p%d = ", j, j);
    scanf("%d", &trabajos[j-1].p);
    printf("Introduce la fecha lmite del trabajo %d:      d%d = ", j, j);
    scanf("%d", &trabajos[j-1].d);
    trabajos[j-1].n_t = j - 1;
    trabajos[j-1].flag = 1;
  }
}


/***************************************************************************/
/* Funcion que lee por fichero los datos para los ejercicios */
void Fichero(void) {
register int j;

  fscanf(fichero, "%d", &n);
  Memoria();
  for (j = 0; j < n; j++) {
    fscanf(fichero, "%d %d", &trabajos[j].p, &trabajos[j].d);
    trabajos[j].n_t = j;
    trabajos[j].flag = 1;
  }
  fclose(fichero);
}

/***************************************************************************/
/* Procedimiento que resuelve el ejercicio 1 */
void Ejercicio1(void) {
register int j;
char string[5];

  /* calculamos la fecha de finalizacion */
  trabajos[0].c = trabajos[0].p;
  for (j = 1; j < n; j++)
    trabajos[j].c = trabajos[j-1].c + trabajos[j].p;

  Iniciar();		/* Iniciamos el modo grafico */
  itos(trabajos[n-1].c, string);
  Ejes(1, string);	/* dibujamos los ejes */

  for (j = 0; j < n; j++)
    /* n trabajo, n maquina tiempo inicial, tiempo final */
    Pon_trabajo(j+1, 1,trabajos[j].c - trabajos[j].p ,trabajos[j].c);

  /* Pausa */
  fflush(stdin);
  getch();
  closegraph();
  free(trabajos);
}

/***************************************************************************/
/* Funcin que ordena los trabajos por el mtodo SPT */
void OrdenarSPT(void) {
register int i,j;
TRABAJO aux;

  for (i = 0; i < n-1; i++)
    for (j = n-1; j > i; j--)
      if (trabajos[j-1].p > trabajos[j].p) {
	aux = trabajos[j-1];
	trabajos[j-1] = trabajos[j];
	trabajos[j] = aux;
      }
}

/***************************************************************************/
/* Procedimiento que resuelve el ejercicio 2. Si ver vale 0, no se muestran
los datos por pantalla. */
void Ejercicio2(int ver) {
register int j;
char string[5];
int contador = 3;		/* contador de lineas */

  /* [C,F,W,L] */
  start = clock();	/* iniciamos el algoritmo */
/* ordenamos el vector de los trabajos por su pj */
  OrdenarSPT();
  /* calculamos los tiempos de finalizacion de los trabajos ya ordenados */
  trabajos[0].c = trabajos[0].p;
  v[0] = trabajos[0].c;
  v[3] = (trabajos[0].c - trabajos[0].d);	/* Lj = Cj - dj */
  v[2] = (trabajos[0].c - trabajos[0].p);	/* wj = Cj - pj */
  for (j= 1; j < n; j++) {
    trabajos[j].c = trabajos[j-1].c + trabajos[j].p;
    /* calculamos la suma de cj, lj y wj */
    v[0] += trabajos[j].c;
    v[3] += (trabajos[j].c - trabajos[j].d);	/* Lj = Cj - dj */
    v[2] += (trabajos[j].c - trabajos[j].p);	/* wj = Cj - pj */
  }
  /* fj = cj - rj y sabemos que rj = 0 */
  v[1] = v[0];
  /* calculamos las medias */
  v[0] /= n;
  v[1] /= n;
  v[2] /= n;
  v[3] /= n;
  if (v[3] < 0)
    v[3] = 0;
  end = clock();	/* finalizamos el algoritmo */

  /* visualizamos los resultados */
  if (ver) {
    clrscr();
    printf("La planificacin ptima es la siguiente:\n");
    printf("\nTrabajo   T. inicio   T.finalizacin   Fecha lmite");
    for (j = 0; j < n; j++) {
      printf("\n  %3d        %4d          %4d           %4d",
	trabajos[j].n_t+1,trabajos[j].c-trabajos[j].p, trabajos[j].c, trabajos[j].d);
      contador++;
      /* para mostrar pantalla a pantalla */
      if (contador == 23) {
	contador = 1;
	Pausa();
	clrscr();
	printf("Trabajo   T. inicio   T.finalizacin   Fecha lmite");
      }
    }

    /* visualizamos los resultados de las funciones objetivo */
    gotoxy(58,3);
    cprintf("F medio = %.2f", v[1]);
    gotoxy(58,4);
    cprintf("W medio = %.2f", v[2]);
    gotoxy(58,5);
    cprintf("C medio = %.2f", v[0]);
    gotoxy(58,6);
    cprintf("L medio = %.2f", v[3]);

    fflush(stdin);
    getch();
    clrscr();
    if (S_N("Desea ver la solucin en formato grfico?") == 'S') {
      Iniciar();		/* Iniciamos el modo grafico */
      itos(trabajos[n-1].c, string);
      Ejes(1, string);	/* dibujamos los ejes */
      for (j = 0; j < n; j++)
	/* n trabajo, n maquina tiempo inicial, tiempo final */
	Pon_trabajo(trabajos[j].n_t+1, 1,trabajos[j].c-trabajos[j].p ,trabajos[j].c);
      /* Pausa */
      fflush(stdin);
      getch();
      closegraph();
    }
    free(trabajos);
  } /* if (ver) */
}

/***************************************************************************/
/* Funcin que ordena los trabajos por el mtodo EDD */
void OrdenarEDD(void) {
register int i,j;
TRABAJO aux;

  for (i = 0; i < n-1; i++)
    for (j = n-1; j > i; j--)
      if (trabajos[j-1].d > trabajos[j].d) {
	aux = trabajos[j-1];
	trabajos[j-1] = trabajos[j];
	trabajos[j] = aux;
      }
}

/***************************************************************************/
/* Procedimiento que resuelve el ejercicio 3 */
void Ejercicio3(int ver) {
register int j;
char string[5];
int Lj;
int contador = 3;

  start = clock();
/* ordenamos el vector de los trabajos por su dj */
  OrdenarEDD();
  /* calculamos los tiempos de finalizacion de los trabajos ya ordenados */
  trabajos[0].c = trabajos[0].p;
  Lmax = trabajos[0].c-trabajos[0].d;
  for (j= 1; j < n; j++) {
    trabajos[j].c = trabajos[j-1].c + trabajos[j].p;
    Lj = trabajos[j].c-trabajos[j].d;	/* demora para cada trabajo */
    if (Lmax < Lj)
      Lmax = Lj;
  }
  if (Lmax < 0)
    Lmax = 0;
  end = clock();

  if (ver) {
    /* visualizamos los resultados */
    clrscr();
    printf("La planificacin ptima es la siguiente:\n");
    printf("\nTrabajo   T.inicio(Sj)   T.finalizacin(Cj)   Fecha lmite(dj)   Demora(Lj)");
    for (j = 0; j < n; j++) {
      Lj = trabajos[j].c-trabajos[j].d;	/* demora para cada trabajo */
      if (Lj < 0)
	Lj = 0;
      printf("\n  %3d         %4d             %4d                %4d                %d",
      trabajos[j].n_t+1,trabajos[j].c-trabajos[j].p, trabajos[j].c, trabajos[j].d, Lj);
      contador++;
      /* para mostrar pantalla a pantalla */
      if (contador == 23) {
	contador = 1;
	Pausa();
	clrscr();
	printf("Trabajo   T.inicio(Sj)   T.finalizacin(Cj)   Fecha lmite(dj)   Demora(Lj)");
      }
    }

   /* mostramos el valor de la funcion objetivo */
    gotoxy(1, wherey()+2);
    textcolor(WHITE);
    cprintf("Lmax = %d", Lmax);
    textcolor(LIGHTGRAY);

    if (S_N("Desea ver la solucin en formato grfico?") == 'S') {
      Iniciar();		/* Iniciamos el modo grafico */
      itos(trabajos[n-1].c, string);
      Ejes(1, string);	/* dibujamos los ejes */
      for (j = 0; j < n; j++)
	/* n trabajo, n maquina tiempo inicial, tiempo final */
	Pon_trabajo(trabajos[j].n_t+1, 1,trabajos[j].c-trabajos[j].p ,trabajos[j].c);
      /* Pausa */
     fflush(stdin);
     getch();
     closegraph();
    }
    free(trabajos);
  } /* if ver */
}

/***************************************************************************/
/* Funcin que resuelve el ejercicio 4, 1 || SUj por el algoritmo de Moore
y Hodgson */
void Ejercicio4(int ver) {
int tardio = 1;		/* indica si quedan trabajos tardios */
register int j,i;
int p;		/* tiempo de proceso actual */
int l,  	/* trabajo a sacar de la lista con mayor pj */
    max;	/* maximo pj */
int salir;
char string[5];
TRABAJO aux;
int contador = 3;

  u = 0;
  start = clock();
  /* ordenamos el vector de los trabajos por su dj */
  OrdenarEDD();
  while (tardio) {
    /* buscamos el primer trabajo tardio */
    salir = 0;	/* FALSE */
    p = 0;
    for (j = 0; (!salir) && (j < n); j++)
      if (trabajos[j].flag) {
	p += trabajos[j].p;
	if (p > trabajos[j].d)
	  salir = 1;
      }
    /* el trabajo j-esimo es el primero tardio */
    if (j == n)		/* no existen trabajos tardios */
      tardio = 0;
    else {
      /* buscamos el trabajo entre [1] y [j] con mayor tiempo de proceso */
      max = 0;
      for (i = j-1; i >= 0; i--)
	if ((trabajos[i].p > max) && (trabajos[i].flag)) {
	  l = i;
	  max = trabajos[i].p;
	}
	/* el trabajo l-esimo es el que tiene pj mayor y lo sacamos de la lista
	poniendo su flag = 0 */
	trabajos[l].flag = 0;
      }
    }
  /* calculamos los tiempos de finalizacion de los trabajos ya ordenados */
  p = 0;
  for (j= 0; j < n; j++)
    if (trabajos[j].flag) {
      p += trabajos[j].p;
      trabajos[j].c = p;
      if (trabajos[j].c > trabajos[j].d)
	u++;
    }
  for (j= 0; j < n; j++)
    if (!trabajos[j].flag) {
      p += trabajos[j].p;
      trabajos[j].c = p;
      if (trabajos[j].c > trabajos[j].d)
	u++;
    }
  end = clock();
  if (ver) {
    /* resultados */
    clrscr();
    printf("La planificacin ptima es la siguiente:\n");
    printf("\nTrabajo   T.inicio(Sj)   T.finalizacin(Cj)   Fecha lmite(dj)");
    for (j = 0; j < n; j++)
      if (trabajos[j].flag) {
	printf("\n  %3d         %4d             %4d                %4d",
	trabajos[j].n_t+1,trabajos[j].c-trabajos[j].p, trabajos[j].c, trabajos[j].d);
	contador++;
	/* para mostrar pantalla a pantalla */
	if (contador == 23) {
	  contador = 1;
	  Pausa();
	  clrscr();
	  printf("Trabajo   T.inicio(Sj)   T.finalizacin(Cj)   Fecha lmite(dj)");
	}

      }
    for (j = 0; j < n; j++)
      if (!trabajos[j].flag) {
	printf("\n  %3d         %4d             %4d                %4d",
	trabajos[j].n_t+1,trabajos[j].c-trabajos[j].p, trabajos[j].c, trabajos[j].d);
	contador++;
	/* para mostrar pantalla a pantalla */
	if (contador == 23) {
	  contador = 1;
	  Pausa();
	  clrscr();
	  printf("Trabajo   T.inicio(Sj)   T.finalizacin(Cj)   Fecha lmite(dj)");
	}
      }
    /* mostramos el valor de la funcion objetivo */
    gotoxy(1, wherey()+2);
    textcolor(WHITE);
    cprintf("Uj = %d", u);
    textcolor(LIGHTGRAY);
    Pausa();


    if (S_N("Desea ver la solucin en formato grfico?") == 'S') {
      Iniciar();		/* Iniciamos el modo grafico */
      itos(p, string);
      Ejes(1, string);	/* dibujamos los ejes */
      for (j = 0; j < n; j++)
       /* n trabajo, n maquina tiempo inicial, tiempo final */
	Pon_trabajo(trabajos[j].n_t+1, 1,trabajos[j].c-trabajos[j].p ,trabajos[j].c);

      /* Pausa */
      fflush(stdin);
      getch();
      closegraph();
    }
    free(trabajos);
  } /* if (ver) */
}

/***************************************************************************/
/* Funcion que asigna memoria a los vectores p y d*/
void Memoria2(void) {

  /* memoria para los pj */
  p = (int *) calloc(n, sizeof(int));
  if (p == NULL) {
    printf("\n\nERROR, no se ha podido asignar memoria al vector pj.");
    exit(1);
  }

  /* memoria para los dj */
  d = (int *) calloc(n, sizeof(int));
  if (d == NULL) {
    printf("\n\nERROR, no se ha podido asignar memoria al vector dj.");
    exit(1);
  }

}
/***************************************************************************/
/* Funcin que genera los trabajos aleatorios para los valores de TF y RDD
indicados en tf y rdd respectivamente y los almacena en el fichero adecuado. */
void Aleatorios(float tf, float rdd, int P) {
register int j;
int sup, inf;	/* limites superior e inferior para el intervalo de los dj */
float x;

  inf = P * (1 - tf - rdd/2);
  sup = P * (1 - tf + rdd/2);
  /* generamos los dj */
  for (j = 0; j < n; j++) {
    d[j] = random(sup-inf+1) + inf;    /* numeros entre inf y sup */
    if (d[j] < 0)
      d[j] = 0;
  }

  /* escritura en fichero */
  fprintf(fichero, "%d\n", n);
  for (j = 0; j < n; j++)
    fprintf(fichero, "%d %d\n", p[j], d[j]);
  fclose(fichero);
}

/***************************************************************************/
/* Funcion que crea el fichero para almecenar los datos con el nombre
adecuado si el flag est a TRUE (1). Si no lo abre para lectura. */
void Preparar_fichero(int numero, int flag) {
char nombre[12];	/* nombre del fichero */
char string[3];
register int i;
int longitud;

  /* ponemos el n de trabajos */
  itos(n, string);
  strcpy(nombre, string);
  longitud = strlen(nombre);
  nombre[longitud] = '-';
  /* pasamos la semilla */
  nombre[longitud+1] = semilla + 48;
  nombre[longitud+2] = '-';
  nombre[longitud+3] = '\0';
  /* pasamos el n de problema */
  longitud = strlen(nombre);
  itos(numero, string);
  for (i = 0; i <= strlen(string); i++)
    nombre[longitud+i] = string[i];
  /* aadimos la extension .in */
  longitud = strlen(nombre);
  nombre[longitud] = '.';
  nombre[longitud+1] = 'i';
  nombre[longitud+2] = 'n';
  nombre[longitud+3] = '\0';

  if (flag) {
    if (!(fichero = fopen(nombre, "w"))) {
      printf("\n\nERROR, no se ha podido crear el fichero %s", nombre);
      exit(1);
    }
  }
  else
    if (!(fichero = fopen(nombre, "r"))) {
      printf("\n\nERROR, no se ha podido abrir el fichero %s", nombre);
      exit(1);
    }
}

/***************************************************************************/
/* Mdulo que genera trabajos para los algoritmos de planificacin.
	Los trabajos vienen caracterizados por su pj y dj. Estos valores se
generarn de forma aleatoria.

pj U[1,100]
dj U[ P*(1-TF-RDD/2) , P*(1-TF+RDD/2) ]
siendo P = Spj
       TF {0.2, 0.4, 0.6, 0.8, 1}
       RDD {0.2, 0.4, 0.6, 0.8}

	Como para cada valor de n (n de trabajos a generar) vamos a tener
20 problemas diferentes (5 TF * 4 RDD) se irn guardando en ficheros donde
el n detrs del ltino guin en el nombre del fichero indica el n de problema.
El nombre del fichero ser de la siguiente forma: 30-2-5.in --> Esto indica
que el n de trabajos es 30, la semilla usada es 2 y es el problema 5
(correspondera en este caso a TF= 0.4 y RDD= 0.2). En el fichero se
almacenar de la forma:
    n de trabajos
    p1 d1
    p2 d2
    .
    .
    .
    pn dn
siendo todos los valores de tipo int.
*/
void Ejercicio5(void) {
register int i,j, k;
int problema;	/* n del problema actual */
int P;		/* suma de los tiempos de proceso */

  clrscr();
  printf("Introduce la semilla (0-9): ");
  scanf("%d", &semilla);
  while((semilla < 0) || (semilla > 9)) {
    printf("Introduce la semilla (0-9): ");
    scanf("%d", &semilla);
  }
  srand(semilla);

  printf("\nSe van a generar problemas para todos los valores de n.\n \
Esto llevar algo de tiempo.");
  if (S_N("Desea continuar?") == 'S')
    for (k = 0; k < 11; k++) {		/* variacion n */
      n = num[k];
      Memoria2();
      problema = 1;
      P = 0;
      /* generamos los pj */
	for (j = 0; j < n; j++) {
	  p[j] = random(100) + 1;	/* numeros entre 1 y 100 */
	  P += p[j];
	}
      for (i = 0; i < 5; i++) 		/* variacion TF */
	for (j = 0; j < 4; j++) {	/* variacion RDD */
	  Preparar_fichero(problema, 1);
	  printf("\nCreando problema n %d...", 20 * k + problema);
	  problema++;
	  Aleatorios(TF[i], RDD[j], P);
	}

    free(p);
    free(d);
    } /* end k */
}

/***************************************************************************/
/* Funcin que muestra en un fichero una tabla con todos los resultados
obtenidos al aplicar los trabajos generados aleatoriamente sobre los
algoritmos de los ejercicios 2, 3 y 4. */
void Ejercicio6(void) {
int problema ;		/* indica el problema actual */
register int i,j,k;

  /* escribimos el encabezado */
  fprintf(out, "  n   TF   RDD   funcin objetivo     tiempo ");
  for (k = 0; k < 11; k++) {
    problema = 1;
    n = num[k];
    for (i= 0; i < 5; i++)
      for (j = 0; j < 4; j++) {
	/* abrimos el fichero */
	Preparar_fichero(problema, 0);
	problema++;
	/* leemos los datos */
	Fichero();
	/* problemas equivalentes */
	Ejercicio2(0);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||C_ = %4.2f       %1.4f",
			n ,TF[i], RDD[j], v[0], (end - start) / CLK_TCK);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||F_ = %4.2f       %1.4f",
			n ,TF[i], RDD[j], v[1], (end - start) / CLK_TCK);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||W_ = %4.2f       %1.4f",
			n ,TF[i], RDD[j], v[2], (end - start) / CLK_TCK);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||L_ = %4.2f        %1.4f",
			 n ,TF[i], RDD[j], v[3], (end - start) / CLK_TCK);
	/* 1||Lmax */
	Ejercicio3(0);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||Lmax = %3d       %1.4f",
			n ,TF[i], RDD[j], Lmax, (end - start) / CLK_TCK);
	/* 1||Uj */
	Ejercicio4(0);
	fprintf(out, "\n %3d  %1.1f  %1.1f    1||Uj = %3d        %1.4f",
			   n ,TF[i], RDD[j], u, (end - start) / CLK_TCK);
        printf("\nResolviendo problema %d...", 20 * k + problema);
	free(trabajos);
      }
  }
  fclose(out);
}


/***************************************************************************/
int main(void) {
unsigned opc;

  do{
    opc = Menu(opciones);
    Menu_principal(opc);
  }while (opc != 0);

  return(0);
}